﻿Imports Microsoft.VisualStudio.TestTools.UnitTesting
Imports Should.Fluent

Namespace DocumentCache.Tests
    <TestClass>
    Public Class DocumentDatabaseOutputCacheProviderTests
        Private _provider As DocumentDatabaseOutputCacheProvider

        <TestInitialize>
        Public Sub BeforeTest()
            _provider = New DocumentDatabaseOutputCacheProvider
        End Sub

        <TestMethod>
        Public Sub should_save_new_object_to_cache()
            Dim key = "12434"

            _provider.Add(key, "Hello MongoDB", Date.UtcNow.AddSeconds(100))
            Dim obj = _provider.Get(key)

            obj.Should().Equal("Hello MongoDB")

            _provider.Remove(key)
        End Sub

        <TestMethod>
        Public Sub should_get_existing_object_from_cache()
            Dim key = "98762"

            _provider.Add(key, "Saved Item", Date.UtcNow.AddSeconds(100))
            Dim obj = _provider.Get(key)

            obj.Should().Equal("Saved Item")

            _provider.Remove(key)
        End Sub

        <TestMethod>
        Public Sub should_not_get_non_existent_object_from_cache()
            Dim key = "90877000"

            Dim obj = _provider.Get(key)

            obj.Should().Be.Null()
        End Sub

        <TestMethod>
        Public Sub should_not_get_expired_object_from_cache()
            Dim key = "98734234"

            _provider.Add(key, "This has expired already!", Date.UtcNow.AddSeconds(-10))
            Dim obj = _provider.Get(key)

            obj.Should().Be.Null()
        End Sub

        <TestMethod>
        Public Sub add_should_not_update_existing_item_in_cache()
            Dim key = "9873"

            _provider.Add(key, "Existing Item", Date.UtcNow.AddSeconds(100))
            _provider.Add(key, "Updated Item", Date.UtcNow.AddSeconds(100))
            Dim obj = _provider.Get(key)

            obj.Should().Equal("Existing Item")
        End Sub

        <TestMethod>
        Public Sub add_should_update_expired_item_in_cache()
            Dim key = "564536787"

            _provider.Add(key, "This has expired already!", Date.UtcNow.AddSeconds(-10))
            _provider.Add(key, "This hasn't", Date.UtcNow.AddSeconds(10))
            Dim obj = _provider.Get(key)

            obj.Should().Equal("This hasn't")
        End Sub

        <TestMethod>
        Public Sub set_should_update_existing_object_from_cache()
            Dim key = "45645"

            _provider.Add(key, "New Document", Date.UtcNow.AddSeconds(100))
            _provider.Set(key, "Updated Document", Date.UtcNow.AddSeconds(100))
            Dim obj = _provider.Get(key)

            obj.Should().Equal("Updated Document")

            _provider.Remove(key)
        End Sub

        <TestMethod>
        Public Sub should_remove_valid_object_from_cache()
            Dim key = "08977"

            _provider.Add(key, "Document to be deleted", Date.UtcNow.AddSeconds(100))
            _provider.Remove(key)

            Dim obj = _provider.Get(key)

            obj.Should().Be.Null()
        End Sub

        <TestCleanup>
        Public Sub Cleanup()
            _provider.Dispose()
        End Sub
    End Class
End Namespace
