﻿Imports System.IO
Imports System.Runtime.Serialization.Formatters.Binary
Imports System.Security.Cryptography
Imports System.Text
Imports System.Web.Caching
Imports MongoDB

Namespace DocumentCache
    Public Class DocumentDatabaseOutputCacheProvider
        Inherits OutputCacheProvider
        Implements IDisposable
        Private ReadOnly _mongo As Mongo
        Private ReadOnly _cacheItems As IMongoCollection(Of CacheItem)

        Public Sub New()
            _mongo = New Mongo
            _mongo.Connect()

            Dim store = _mongo.GetDatabase("OutputCacheDB")
            _cacheItems = store.GetCollection(Of CacheItem)()
        End Sub

        Public Overrides Function [Get](ByVal key As String) As Object
            Debug.WriteLine(String.Format("Cache.Get({0})", key))

            key = MD5(key)

            Dim cacheItem = _cacheItems.FindOne(New With {Key ._id = key})

            If cacheItem IsNot Nothing Then
                If cacheItem.Expiration.ToUniversalTime() <= Date.UtcNow Then
                    _cacheItems.Remove(cacheItem)
                Else
                        Return Deserialize(cacheItem.Item)
                End If
            End If

            Return Nothing
        End Function

        Public Overrides Function Add(ByVal key As String,
                                      ByVal entry As Object,
                                      ByVal utcExpiry As Date) As Object
            Debug.WriteLine("Cache.Add({0}, {1}, {2})", key, entry, utcExpiry)

            key = MD5(key)

            If utcExpiry = Date.MaxValue Then
                utcExpiry = Date.UtcNow.AddMinutes(5)
            End If

            Dim item = _cacheItems.FindOne(New With {Key ._id = key})

            If item IsNot Nothing Then
                If item.Expiration.ToUniversalTime() <= Date.UtcNow Then
                    _cacheItems.Remove(item)
                Else
                    Return Deserialize(item.Item)
                End If
            End If

            _cacheItems.Insert(New CacheItem With
                               {.Id = key,
                                .Item = Serialize(entry),
                                .Expiration = utcExpiry})

            Return entry
        End Function

        Public Overrides Sub [Set](ByVal key As String,
                                   ByVal entry As Object,
                                   ByVal utcExpiry As Date)
            Debug.WriteLine("Cache.Set({0}, {1}, {2})", key, entry, utcExpiry)

            key = MD5(key)

            Dim item = _cacheItems.FindOne(New With {Key ._id = key})

            If item IsNot Nothing Then
                item.Item = Serialize(entry)
                item.Expiration = utcExpiry
                _cacheItems.Save(item)
            Else
                _cacheItems.Insert(New CacheItem With
                                   {.Id = key,
                                    .Item = Serialize(entry),
                                    .Expiration = utcExpiry})
            End If
        End Sub

        Public Overrides Sub Remove(ByVal key As String)
            Debug.WriteLine("Cache.Remove({0})", key)

            key = MD5(key)

            _cacheItems.Remove(New With {Key ._id = key})
        End Sub

        Private Shared Function MD5(ByVal value As String) As String
            Dim cryptoServiceProvider = New MD5CryptoServiceProvider
            Dim bytes = Encoding.UTF8.GetBytes(value)
            Dim builder = New StringBuilder

            bytes = cryptoServiceProvider.ComputeHash(bytes)

            For Each b In bytes
                builder.Append(b.ToString("x2").ToLower())
            Next b

            Return builder.ToString()
        End Function

        Private Shared Function Serialize(ByVal entry As Object) As Byte()
            Dim formatter = New BinaryFormatter
            Dim stream = New MemoryStream
            formatter.Serialize(stream, entry)

            Return stream.ToArray()
        End Function

        Private Shared Function Deserialize(ByVal serializedEntry() As Byte) As Object
            Dim formatter = New BinaryFormatter
            Dim stream = New MemoryStream(serializedEntry)

            Return formatter.Deserialize(stream)
        End Function

        Public Sub Dispose() Implements IDisposable.Dispose
            _mongo.Disconnect()
            _mongo.Dispose()
        End Sub
    End Class
End Namespace
